#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "tft.h"
#include "itdb02.h"
#include "gui.h"

void ClearScreen(void)
{
  setColor(GUI_COLOR.br, GUI_COLOR.bg, GUI_COLOR.bb);
  if(orient==LANDSCAPE)
  {
      fillRect(1, 14, 318, 225);
  }
  else
  {
      fillRect(1, 14, 238, 306);
  }
}

void DrawScreen(void)
{
  ClearScreen();
  if(orient==LANDSCAPE)
  {
      //Draw top and bottom field
      setFont(SmallFont);
      setColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      fillRect(0, 0, 320, 13);
      fillRect(0, 226, 320, 239);
      // Draw main frame
      setBackColor(0, 0, 0);
      setColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      drawRect(0, 13, 319, 225);
  }
  else
  {
      //Draw top and bottom field
      setFont(SmallFont);
      setColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      fillRect(0,   0, 239, 13);
      fillRect(0, 305, 239, 318);
      // Draw main frame
      setBackColor(0, 0, 0);
      setColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      drawRect(0, 13, 239, 307);
  }
}

void UpdateScreen(void)
{
      setFont(SmallFont);
      setBackColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      setColor(GUI_COLOR.tr, GUI_COLOR.tg, GUI_COLOR.tb);
      print(GUI_TOPLINE,CENTER,0,0);
      /// Top zeile ende

      /// hier rein, was in die bottom zeile gehoert
#ifndef DEBUG
      setColor(255,255,255);
      if(orient==LANDSCAPE)
      {
	print(GUI_BOTTOMLINE,CENTER,227,0);
      }
      else
      {
	print(GUI_BOTTOMLINE,CENTER,307,0);
      }
#endif
}

void gui_top_line(char *line)
{
  sprintf(GUI_TOPLINE,line);
  DrawScreen();
  UpdateScreen();
}

void gui_bottom_line(char *line)
{
  sprintf(GUI_BOTTOMLINE,line);
  DrawScreen();
  UpdateScreen();
}

void gui_style(int theme)
{
  NUM_ELEMENTS=0;
  GUI_STYLE=theme;
  switch (theme)
  {
    case STYLE_HIDDEN:
      /// Buttons
      // Background black
      GUI_COLOR.br=0;
      GUI_COLOR.bg=0;
      GUI_COLOR.bb=0;
      // buttons are black
      GUI_COLOR.r=0;
      GUI_COLOR.g=0;
      GUI_COLOR.b=0;
      // frame is black
      GUI_COLOR.fr=0;
      GUI_COLOR.fg=0;
      GUI_COLOR.fb=0;
      // active frame is black
      GUI_COLOR.ar=0;
      GUI_COLOR.ag=0;
      GUI_COLOR.ab=0;
      // text is black
      GUI_COLOR.tr=0;
      GUI_COLOR.tg=0;
      GUI_COLOR.tb=0;
      break;
      
    case STYLE_BLUE:
      /// Buttons
      // Background black
      GUI_COLOR.br=0;
      GUI_COLOR.bg=0;
      GUI_COLOR.bb=0;
      // buttons are blue
      GUI_COLOR.r=60;
      GUI_COLOR.g=60;
      GUI_COLOR.b=200;
      // frame is white
      GUI_COLOR.fr=255;
      GUI_COLOR.fg=255;
      GUI_COLOR.fb=255;
      // active frame is red
      GUI_COLOR.ar=255;
      GUI_COLOR.ag=0;
      GUI_COLOR.ab=0;
      // text is white
      GUI_COLOR.tr=255;
      GUI_COLOR.tg=255;
      GUI_COLOR.tb=255;
      break;
      
    case STYLE_RED:
      /// Buttons
      // Background yellow
      GUI_COLOR.br=255;
      GUI_COLOR.bg=255;
      GUI_COLOR.bb=0;
      // buttons are red
      GUI_COLOR.r=255;
      GUI_COLOR.g=0;
      GUI_COLOR.b=0;
      // frame is green
      GUI_COLOR.fr=0;
      GUI_COLOR.fg=102;
      GUI_COLOR.fb=0;
      // active frame is yellow
      GUI_COLOR.ar=255;
      GUI_COLOR.ag=255;
      GUI_COLOR.ab=0;
      // text is white
      GUI_COLOR.tr=255;
      GUI_COLOR.tg=255;
      GUI_COLOR.tb=255;
      break;
      
    case STYLE_GREEN:
      /// Buttons
      // Background white
      GUI_COLOR.br=255;
      GUI_COLOR.bg=255;
      GUI_COLOR.bb=255;
      // buttons are green
      GUI_COLOR.r=0;
      GUI_COLOR.g=102;
      GUI_COLOR.b=0;
      // frame is red
      GUI_COLOR.fr=255;
      GUI_COLOR.fg=0;
      GUI_COLOR.fb=0;
      // active frame is black
      GUI_COLOR.ar=0;
      GUI_COLOR.ag=0;
      GUI_COLOR.ab=0;
      // text is yellow
      GUI_COLOR.tr=255;
      GUI_COLOR.tg=255;
      GUI_COLOR.tb=0;
      break;
    }
    DrawScreen();
    UpdateScreen();
}

void guiShowElement(int num)
{
  switch(GUI_ELEMENT[num].type)
  {
    case gui_type_button:
      setFont(BigFont);
      setColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      fillRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
      drawRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setBackColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      setColor(GUI_COLOR.tr,GUI_COLOR.tg,GUI_COLOR.tb);
      print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),(GUI_ELEMENT[num].y+(GUI_ELEMENT[num].h/2))-((cfont.y_size)/2),0);
      break;
    case gui_type_round_button:
      setFont(BigFont);
      setColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      fillRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
      drawRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setBackColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      setColor(GUI_COLOR.tr,GUI_COLOR.tg,GUI_COLOR.tb);
      print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),(GUI_ELEMENT[num].y+(GUI_ELEMENT[num].h/2))-((cfont.y_size)/2),0);
      break;
    case gui_type_small_button:
      setFont(SmallFont);
      setColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      fillRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
      drawRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setBackColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      setColor(GUI_COLOR.tr,GUI_COLOR.tg,GUI_COLOR.tb);
      if(!strlen(GUI_ELEMENT[num].text2))
	print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),GUI_ELEMENT[num].y+12,0);
      else
	print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),GUI_ELEMENT[num].y+2,0);
      print(GUI_ELEMENT[num].text2,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text2)*cfont.x_size)/2),GUI_ELEMENT[num].y+GUI_ELEMENT[num].h-2-cfont.y_size,0);
      break;
    case gui_type_small_round_button:
      setFont(SmallFont);
      setColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      fillRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
      drawRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
      setBackColor(GUI_COLOR.r,GUI_COLOR.g,GUI_COLOR.b);
      setColor(GUI_COLOR.tr,GUI_COLOR.tg,GUI_COLOR.tb);
      if(!strlen(GUI_ELEMENT[num].text2))
	print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),GUI_ELEMENT[num].y+12,0);
      else
	print(GUI_ELEMENT[num].text,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text)*cfont.x_size)/2),GUI_ELEMENT[num].y+2,0);
      print(GUI_ELEMENT[num].text2,(GUI_ELEMENT[num].x+(GUI_ELEMENT[num].w/2))-((strlen(GUI_ELEMENT[num].text2)*cfont.x_size)/2),GUI_ELEMENT[num].y+GUI_ELEMENT[num].h-2-cfont.y_size,0);
      break;
  }
}

int8_t gui_checkElement(int num, int x, int y)
{
  if(GUI_ELEMENT[num].state == gui_state_inactive)
    return FALSE;
  
  switch (GUI_ELEMENT[num].type)
  {
    case gui_type_round_button:
    case gui_type_small_round_button:
      if( (x>GUI_ELEMENT[num].x) && (x< (GUI_ELEMENT[num].x+GUI_ELEMENT[num].w)) && (y>GUI_ELEMENT[num].y) && (y<(GUI_ELEMENT[num].y+GUI_ELEMENT[num].h)) )
      {
	if(GUI_ELEMENT[num].state == gui_state_active)
	{
	  if(GUI_STYLE!=STYLE_HIDDEN)
	  {
	    setColor(GUI_COLOR.ar,GUI_COLOR.ag,GUI_COLOR.ab);
	    drawRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
	  }
	  if(GUI_ELEMENT[num].callback!=NULL)
	  {
	    GUI_LAST_ACTIVE=num;
	    GUI_ELEMENT[num].callback();
	  }
	  else
	  {
	    if(GUI_STYLE!=STYLE_HIDDEN)
	    {
	      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
	      drawRoundRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
	    }
	  }
	}
      }
      break;
    case gui_type_button:
    case gui_type_small_button:
      if( (x>GUI_ELEMENT[num].x) && (x< (GUI_ELEMENT[num].x+GUI_ELEMENT[num].w)) && (y>GUI_ELEMENT[num].y) && (y<(GUI_ELEMENT[num].y+GUI_ELEMENT[num].h)) )
      {
	if(GUI_ELEMENT[num].state == gui_state_active)
	{
	  if(GUI_STYLE!=STYLE_HIDDEN)
	  {
	    setColor(GUI_COLOR.ar,GUI_COLOR.ag,GUI_COLOR.ab);
	    drawRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
	  }
	  if(GUI_ELEMENT[num].callback!=NULL)
	  {
	    GUI_LAST_ACTIVE=num;
	    GUI_ELEMENT[num].callback();
	  }
	  else
	  {
	    if(GUI_STYLE!=STYLE_HIDDEN)
	    {
	      setColor(GUI_COLOR.fr,GUI_COLOR.fg,GUI_COLOR.fb);
	      drawRect(GUI_ELEMENT[num].x, GUI_ELEMENT[num].y, GUI_ELEMENT[num].x+GUI_ELEMENT[num].w, GUI_ELEMENT[num].y+GUI_ELEMENT[num].h);
	    }
	  }
	}
      }
      break;
    case gui_type_bottom_button:
      if(y>(240-GUI_ELEMENT[num].h))
      {
	if(GUI_ELEMENT[num].state == gui_state_active)
	{
	  if(GUI_ELEMENT[num].callback!=NULL)
	  {
	    GUI_LAST_ACTIVE=num;
	    GUI_ELEMENT[num].callback();
	  }
	}
      }
      break;
  }

  
  return TRUE;
}

void guiSetElementState(int num, int state)
{
  GUI_ELEMENT[num].state=state;
}

void guiNewElement(int type, char *text, char *text2, int x, int y, int w, int h, void (*cb)(void))
{
  if(type==gui_type_thermometer)
  {
    x=x-20;
    y=22;
    w=40;
    h=170;
  }
  GUI_ELEMENT[NUM_ELEMENTS].x=x;
  GUI_ELEMENT[NUM_ELEMENTS].y=y;
  GUI_ELEMENT[NUM_ELEMENTS].w=w;
  GUI_ELEMENT[NUM_ELEMENTS].h=h;
  GUI_ELEMENT[NUM_ELEMENTS].type=type;
  GUI_ELEMENT[NUM_ELEMENTS].state=gui_state_inactive;
  GUI_ELEMENT[NUM_ELEMENTS].callback=cb;
  sprintf(GUI_ELEMENT[NUM_ELEMENTS].text,text);
  sprintf(GUI_ELEMENT[NUM_ELEMENTS].text2,text2);
  NUM_ELEMENTS++;
}

void gui_start(void)
{
  int i;
  
  setFont(SmallFont);
  ClearScreen();
  UpdateScreen();
  for(i=0;i<NUM_ELEMENTS;i++)
  {
    guiShowElement(i);
    guiSetElementState(i, gui_state_active);
  }
}

void gui_refresh(void)
{
  int i;
  
  setFont(SmallFont);
  for(i=0;i<NUM_ELEMENTS;i++)
  {
    guiShowElement(i);
    guiSetElementState(i, gui_state_active);
  }
}

void gui_stop(void)
{
  int i;
  for(i=0;i<NUM_ELEMENTS;i++)
  {
    guiSetElementState(i, gui_state_inactive);
  }
}

void gui_check(int x, int y)
{
  int i;
    for(i=0;i<NUM_ELEMENTS;i++)
    {
      gui_checkElement(i, x, y);
    }
}

