#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "main.h"
#include "io.h"
#include "tft.h"
#include "itdb02.h"


int cx, cy;
int rx[10], ry[10];
float px, py;
int ox, oy;

void drawCrossHair(int x, int y)
{
//  drawRect(x-10, y-10, x+10, y+10);
  drawCircle(x,y,8);
  drawLine(x-10, y, x+10, y);
  drawLine(x, y-10, x, y+11);
}

void readCoordinates(void)
{
  int iter = 20;
  int cnt = 0;
  unsigned long tx=0;
  unsigned long ty=0;
  boolean OK = false;
  
  while (OK == false)
  {
    while(touch_dataAvailable()==false){}
    while( (touch_dataAvailable()==true) && (cnt<iter))
    {
	touch_read();
	tx += TP_X;
	ty += TP_Y;
	cnt++;
    }	
    if(cnt>=iter)
    {
      OK=true;
    }
    else
    {
      tx = 0;
      ty = 0;
      cnt = 0;
    }
  }

  cx = tx/iter;
  cy = ty/iter;

}

void calibrate(int x, int y, int i)
{
  setColor(0,255,0);
  drawCrossHair(x,y);
  readCoordinates();
  setColor(40, 40, 40);
  drawCrossHair(x,y);
  
  rx[i]=cx;
  ry[i]=cy;
  while (touch_dataAvailable() == true)
  {
    touch_read();
  }
}

void waitForTouch(void)
{
  while (touch_dataAvailable() == true)
  {
    touch_read();
  }
  while (touch_dataAvailable() == false) {}
  while (touch_dataAvailable() == true)
  {
   touch_read();
  }
}

void touch_calibrate(void)
{
  int old_orient=orient;
  LCDSetup(ITDB32S, UTFT_MEGA, FALSE, FALSE);
  InitLCD(PORTRAIT);
  clrScr();
  setColor(40, 40, 40);

  drawCrossHair(10,10);
  drawCrossHair(10,160);
  drawCrossHair(10,309);
  drawCrossHair(120,10);
  drawCrossHair(120,160);
  drawCrossHair(120,309);
  drawCrossHair(229,10);
  drawCrossHair(229,160);
  drawCrossHair(229,309);
      
  calibrate(10, 10, 0);
  calibrate(10, 160, 1);
  calibrate(10, 309, 2);
  calibrate(120, 10, 3);
  calibrate(120, 160, 4);
  calibrate(120, 309, 5);
  calibrate(229, 10, 6);
  calibrate(229, 160, 7);
  calibrate(229, 309, 8);

  px = (((float)(rx[0]+rx[1]+rx[2])/3)-((float)(rx[6]+rx[7]+rx[8])/3))/220;
  if (px>=0)
    ox = (((rx[6]+rx[7]+rx[8])/3))-(px*10);
  else
    ox = (((rx[0]+rx[1]+rx[2])/3))+(px*10);

  py = (((float)(ry[0]+ry[3]+ry[6])/3)-((float)(ry[2]+ry[5]+ry[8])/3))/300;
  if (py>=0)
    oy = (((ry[2]+ry[5]+ry[8])/3))-(py*10);
  else
    oy = (((ry[0]+ry[3]+ry[6])/3))+(py*10);

//  waitForTouch();
  clrScr();
  PixSizeX=px;
  PixOffsX=ox;
  PixSizeY=py;
  PixOffsY=oy;
  InitLCD(old_orient);

}
void ITDB02_Touch(byte tclk, byte tcs, byte din, byte dout, byte irq)
{
    T_CLK = tclk;
    T_CS  = tcs;
    T_DIN = din;
    T_DOUT= dout;
    T_IRQ = irq;
}

void InitTouch(byte orientation)
{
    orient = orientation;
    set_io(T_CLK,  OUTPUT);
    set_io(T_CS,   OUTPUT);
    set_io(T_DIN,  OUTPUT);
    set_io(T_DOUT, INPUT);
    set_io(T_IRQ,  INPUT);

    set_io(T_CS,  OUTPUT_HIGH);
    set_io(T_CLK, OUTPUT_HIGH);
    set_io(T_DIN, OUTPUT_HIGH);
    set_io(T_CLK, OUTPUT_HIGH);
}

void touch_WriteData(byte data)
{
	byte temp;
	byte nop;

	temp=data;
	set_io(T_CLK,OUTPUT_LOW);

	for(byte count=0; count<8; count++)
	{
		if(temp & 0x80)
			set_io(T_DIN, OUTPUT_HIGH);
		else
			set_io(T_DIN, OUTPUT_LOW);
		temp = temp << 1; 
		set_io(T_CLK, OUTPUT_LOW);                
		nop++;
		set_io(T_CLK, OUTPUT_HIGH);
		nop++;
	}
}

word touch_ReadData(void)
{
	byte nop;
	word data = 0;
	for(byte count=0; count<12; count++)
	{
		data <<= 1;
		set_io(T_CLK, OUTPUT_HIGH);               
		nop++;
		set_io(T_CLK, OUTPUT_LOW);
		nop++;
		if (get_io(T_DOUT))
			data++;
	}
	return(data);
}

void touch_read(void)
{
	unsigned long tx=0;
	unsigned long ty=0;

	set_io(T_CS,OUTPUT_LOW);                    

	for (int i=0; i<prec; i++)
	{
		touch_WriteData(0x90);        
		set_io(T_CLK,OUTPUT_HIGH);
		set_io(T_CLK,OUTPUT_LOW); 
		ty+=touch_ReadData();

		touch_WriteData(0xD0);      
		set_io(T_CLK,OUTPUT_HIGH);
		set_io(T_CLK,OUTPUT_LOW);
		tx+=touch_ReadData();
	}

	set_io(T_CS,OUTPUT_HIGH);

	TP_X=tx/prec;
	TP_Y=ty/prec;
}

bool touch_dataAvailable(void)
{
  bool avail;
  avail = !get_io(T_IRQ);
  return avail;
}

int touch_getX(void)
{
	int value;

	if (orient == PORTRAIT)
	{
		if (PixSizeX>=0)
		{
			value = 240-((TP_X-PixOffsX)/PixSizeX);
		}
		else
		{
			value = (TP_X-PixOffsX)/-(PixSizeX);
		}
	}
	else
	{
		if (PixSizeY<0)
			value = 320-((TP_Y-PixOffsY)/-PixSizeY);
		else
			value = ((TP_Y-PixOffsY)/PixSizeY);
	}

	if (value < 0)
		value = 0;
	return value;
}

int touch_getY(void)
{
	int value;
	if (orient == PORTRAIT)
	{
		if (PixSizeY<0)
			value = ((TP_Y-PixOffsY)/-PixSizeY);
		else
			value = 320-((TP_Y-PixOffsY)/PixSizeY);
	}
	else
	{
		if (PixSizeX>=0)
		{
			value = 240-((TP_X-PixOffsX)/PixSizeX);
		}
		else
		{
			value = (TP_X-PixOffsX)/-(PixSizeX);
		}
	}

	if (value < 0)
		value = 0;
	return value;  
}

void touch_setPrecision(byte precision)
{
	switch (precision)
	{
		case PREC_LOW:
			prec=1;
			break;
		case PREC_MEDIUM:
			prec=10;
			break;
		case PREC_HI:
			prec=25;
			break;
		case PREC_EXTREME:
			prec=100;
			break;
		default:
			prec=10;
			break;
	}
}