#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "fonts.h"
#include "logo.h"
#include "tft.h"
#include "itdb02.h"
#include "gui.h"

/** ****************************************************************
*	setup
*
*	@Brief Everything that needs to be set during boot
*
*	@Param none
*
*	@Return none
*******************************************************************/
void setup(void)
{
  PixSizeX=-15.2288;
  PixOffsX=251;
  PixSizeY=-11.0622;
  PixOffsY=218;

  LCDSetup(ITDB32S, UTFT_MEGA, FALSE, FALSE);
  InitLCD(PORTRAIT);
  clrScr();
  int y_pos;
  if(orient==PORTRAIT)
  {
    y_pos=180;		/// portrait
  }
  else
  {
    y_pos=260;		/// landscape
    drawBitmap (20, 200, 32, 32, logo,1);
    drawBitmap (270, 200, 32, 32, logo,1);
  }
  setFont(BigFont);
  print("Booting...",0,0,0);
  setFont(SmallFont);
  int line=20;
  
  setColor(255, 255, 255);
  print("Loading hardware drivers",0,line,0);
  line+=12;

  setColor(255, 255, 255);
  print("Touch screen",0,line,0);
  ITDB02_Touch(6,5,4,3,2);
  InitTouch(orient);
  touch_setPrecision(PREC_LOW);
  setColor(0,153,0);
  print("[done]",y_pos,line,0);
  line+=12;
  
  setColor(255, 255, 255);
  print("System ready",0,line,0);
  setFont(BigFont);
  if(orient==PORTRAIT)
  {
    print("Touch screen",CENTER,280,0);
    print("to continue",CENTER,300,0);
  }
  else
  {
    print("Touch screen",CENTER,198,0);
    print("to continue",CENTER,218,0);
  }
  TOUCH_STATE=STATE_BOOTING;
  
}
void gui_callback_calibrate(void)
{
  TOUCH_STATE=STATE_CALIBRATE_BEGIN;
}

void gui_callback_draw(void)
{
  int min_y=14, max_y=225;
  ClearScreen();
//  UpdateScreen();
  guiSetElementState(0, gui_state_inactive);
  guiSetElementState(1, gui_state_inactive);
  setColor(64, 64, 64);
  drawLine(260, min_y, 260, max_y);
  setColor(255, 255, 255);
  fillRect(264, 20, 274, 30);
  setColor(255, 255, 255);
  TOUCH_STATE=STATE_DRAWING;
}


void MenuMain(void);
void MenuSetup(void);
void MenuSetupGui(void);

int style_num=STYLE_BLUE;

void MenuMain(void)
{
  gui_style(style_num);		// Set colors for gui elements and clear all elements
  gui_top_line("MAIN MENU");
  guiNewElement(gui_type_small_round_button,	"Setup",	"Options" ,		5,  20,  80, 32, MenuSetup);
  gui_start();				// show the gui elements
}

void MenuSetup(void)
{
  gui_style(style_num);		// Set colors for gui elements and clear all elements
  gui_top_line("SETUP OPTIONS");
  guiNewElement(gui_type_small_round_button,	"Back to",	"Main menu" ,	5,  20,  80, 32, MenuMain);
  guiNewElement(gui_type_small_round_button,	"Calibrate",	"Touch" ,	5,  60,  80, 32, gui_callback_calibrate);
  guiNewElement(gui_type_small_round_button,	"Setup",	"Screen" ,	5, 100,  80, 32, MenuSetupGui);
  gui_start();				// show the gui elements
}

void MenuOrientationLandscape(void)
{
  if(orient==PORTRAIT)
  {
    InitLCD(LANDSCAPE);
    clrScr();
    InitTouch(orient);
    MenuSetupGui();
  }
}

void MenuOrientationPortrait(void)
{
  if(orient==LANDSCAPE)
  {
    InitLCD(PORTRAIT);
    clrScr();
    InitTouch(orient);
    MenuSetupGui();
  }
}

void MenuStyleBlue(void)
{
  if(style_num!=STYLE_BLUE)
    style_num=STYLE_BLUE;
  MenuSetupGui();
}
void MenuStyleGreen(void)
{
  if(style_num!=STYLE_GREEN)
  style_num=STYLE_GREEN;
  MenuSetupGui();
}
void MenuStyleRed(void)
{
  if(style_num!=STYLE_RED)
  style_num=STYLE_RED;
  MenuSetupGui();
}

void MenuSetupFactory(void)
{
  style_num=STYLE_BLUE;
  if(orient==PORTRAIT)
  {
    MenuOrientationLandscape();
  }
}

void MenuSetupGui(void)
{
  gui_style(style_num);		// Set colors for gui elements and clear all elements
  gui_top_line("SETUP GUI OPTIONS");
  if(orient==LANDSCAPE)
  {
    guiNewElement(gui_type_small_round_button,	"Close",	"",	  	10, 180,  100, 32, MenuSetup);
    guiNewElement(gui_type_small_round_button,	"Reset",	"Settings",	130, 180,  100, 32, MenuSetupFactory);
  }
  else
  {
    guiNewElement(gui_type_small_round_button,	"Close",	"",		 10, 260,  100, 32, MenuSetup);
    guiNewElement(gui_type_small_round_button,	"Reset",	"Settings",	130, 260,  100, 32, MenuSetupFactory);
  }

  guiNewElement(gui_type_small_round_button,	"Orientation",	"Landscape", 10, 20,  100, 32, MenuOrientationLandscape);
  guiNewElement(gui_type_small_round_button,	"Orientation",	"Portrait",  10, 60,  100, 32, MenuOrientationPortrait);

  guiNewElement(gui_type_small_round_button,	"Style"	, "Blue",  130,  20,  100, 32, MenuStyleBlue);
  guiNewElement(gui_type_small_round_button,	"Style"	, "Green", 130,  60,  100, 32, MenuStyleGreen);
  guiNewElement(gui_type_small_round_button,	"Style"	, "Red",   130, 100,  100, 32, MenuStyleRed);


  gui_start();				// show the gui elements
}

#define DIRECTION_LEFT		1
#define DIRECTION_RIGHT		2
#define DIRECTION_UP		3
#define DIRECTION_DOWN		4
int old_x=0;int old_y=0;
int counter=0;
/** ****************************************************************
*	run
*
*	@Brief All functions that are used in polling mode
*
*	@Param none
*
*	@Return none
*******************************************************************/
void run(void)
{
    int x=0,y=0;
  
///
/// @BRIEF: State machine for gui handling
///
    while (touch_dataAvailable() == false)
    {
#ifdef DEBUG      
	setFont(SmallFont);
	setBackColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
	setColor(255, 255, 255);
	if(orient==LANDSCAPE)
	{
	  print("Pen up     ",0,227,0);
	}
	else
	{
	  print("up  ",0,307,0);
	}
#endif
    }
    while (touch_dataAvailable() == true)
    {
#ifdef DEBUG    
      setFont(SmallFont);
      setBackColor(GUI_COLOR.r, GUI_COLOR.g, GUI_COLOR.b);
      setColor(GUI_COLOR.tr, GUI_COLOR.tg, GUI_COLOR.tb);
      if(orient==LANDSCAPE)
      {
	print("Pen down",0,227,0);
	sprintf(tmp_buffer,"x=%3d  y=%3d",x,y);
	print(tmp_buffer,CENTER,227,0);
      }
      else
      {
	print("down",0,307,0);
	sprintf(tmp_buffer,"%3d %3d",x,y);
	print(tmp_buffer,CENTER,307,0);
      }
#endif    
      touch_read();				// Read touch screen
      x=touch_getX();				// Store x value
      y=touch_getY();				// Store y value
      switch(TOUCH_STATE)
      {
	case STATE_BOOTING:
	  MenuMain();				// define gui elements for main menu
	  TOUCH_STATE=STATE_MAINMENU;
	  break;
	case STATE_CALIBRATE_BEGIN:
	  gui_stop();				// stop gui handling
	  ClearScreen();			// clear the inner frame
	  setFont(SmallFont);
	  setBackColor(0,0,0);
	  setColor(255,255,255);
	  print("How to calibrate",CENTER,20,0);
	  if(orient==LANDSCAPE)
	  {
	    print("You will see a screen with a set of",10,60,0);
	    print("cross where the active one is yellow.",10,72,0);
	    print("Touch the active one until it goes",10,84,0);
	    print("back to gray again.",10,96,0);
	    print("You have to do this until the screen",10,108,0);
	    print("clears and the menu comes up.",10,120,0);
	    print("Touch screen to begin",CENTER,140,0);
	  }
	  else
	  {
	    print("You will see a screen with a",10,60,0);
	    print("set of cross where the",10,72,0);
	    print("active one is yellow. Touch",10,84,0);
	    print("the active one until it goes",10,96,0);
	    print("back to gray again.",10,108,0);
	    print("You have to do this until",10,120,0);
	    print("the screen clears and the",10,132,0);
	    print("menu comes up again.",10,144,0);
	    print("Touch screen to begin",CENTER,160,0);
	  }
	  TOUCH_STATE=STATE_CALIBRATE;
	  break;
	case STATE_CALIBRATE:
	  touch_setPrecision(PREC_HI);
	  touch_calibrate();			// do the calibration
	  touch_setPrecision(PREC_LOW);
	  DrawScreen();
	  gui_start();				// restart gui handling
	  TOUCH_STATE=STATE_MAINMENU;
	  break;
	case STATE_MAINMENU:
	  touch_read();				// Read touch screen
	  gui_check(x, y);
	  break;
      }
    } 
}


/** ****************************************************************
*	main
*
*	@Brief program start and endless loop
*	@Brief everything else will be handeled in setup() and run()
*
*	@Param none
*
*	@Return none
*******************************************************************/
int main(void)
{
  setup();

  while(1)
  {
    run();
  }
    
  return 0;
}


